#!/bin/bash
#
# Script to install the linaro gcc distribution for Linux
#
# Native parts will go in nativeInstallDir=_sysroot/${compiler_loc} while
# target parts will go normally in installDir=_install
#

if [ -r "$PLATFORM_LOC/scripts/utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/utils.sh"
else
	source "$script_lib_dir/utils.sh"
fi

SRC_LOC=${SRC_LOC%%;*} # To suppress ;unpack=false for example...
SRC_TARBALL=${SRC_LOC##*/}
SRC_NAME=${SRC_TARBALL%.tar.*}
nativePrefix="$nativeInstallDir/${compiler_loc#/}"
compiler_prefix=$CROSS_PREFIX

# First cleanup everything

log_exec rm -rf $SRC_NAME $installDir/* $nativeInstallDir
log_exec tar xf $SRC_TARBALL
log_exec mkdir -p $installDir
log_exec mkdir -p $nativePrefix/usr

# Install the cross-gcc script and symbolic links
log_exec mkdir -p $nativePrefix/bin
log_exec cp cross-gcc $nativePrefix/bin
sed -i -- "s/@COMPILER_PREFIX@/$compiler_prefix/g" $nativePrefix/bin/cross-gcc
log_exec chmod a+x $nativePrefix/bin/cross-gcc

log "Then create links to cross-gcc from ${arch_compiler_prefix}- tools"
for tool in $SRC_NAME/bin/${compiler_prefix}-*; do
	if [ -r $tool ]; then
		tool_name=${tool##*/${compiler_prefix}-}
		ln -nfs cross-gcc $nativePrefix/bin/${arch_compiler_prefix}-${tool_name}
	fi
done

#
# Move the C library and include files to $installDir
#
#
# It would be nice to move all files that are installed as part of the C library package 
# only to the target sysroot; however, when compiling some components, like the linux
# kernel, we need to use directly the cross-compiler itself without going through our
# wrapper scripts which are really meant only for user-mode programs. 
# However, for programs like the kernel and bootloaders, you can't use any C libraries
# except the static libraries bundled with the compiler, so we just duplicate these.
#

# Copy target files to $installDir (you will have here executables going
# in /sbin, /usr/bin, /usr/sbin as well as libraries in /lib and /usr/lib etc.
log_exec mv $SRC_NAME/$compiler_prefix/libc/* $installDir

# Note that the content of $SRC_NAME/$compiler_prefix/lib is included
# in $SRC_NAME/$compiler_prefix/libc/lib, so we can suppress it as well as
# $SRC_NAME/$compiler_prefix/libc which is now empty
log_exec rm -rf $SRC_NAME/$compiler_prefix/lib $SRC_NAME/$compiler_prefix/libc

log Create default /etc/ld.so.conf
mkdir -p $installDir/etc/ld.so.conf.d
touch $installDir/etc/ld.so.conf.d/$compiler_prefix.conf
echo "/lib" >> $installDir/etc/ld.so.conf.d/$compiler_prefix.conf
echo "/usr/lib" >> $installDir/etc/ld.so.conf.d/$compiler_prefix.conf

# Copy GCC include files, first in the native sysroot, then in $installDir
# Note copy_tree_excluding will create hard links instead of copying files
copy_tree_excluding $SRC_NAME/$compiler_prefix/include/c++/${GCC_VERSION} $nativePrefix/$compiler_prefix/usr/include/c++/${GCC_VERSION}
copy_tree_excluding $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION} $nativePrefix/usr/lib/gcc/$compiler_prefix/${GCC_VERSION}

log_exec mkdir -p $installDir/usr/include
log_exec mv $SRC_NAME/$compiler_prefix/include/c++/${GCC_VERSION} $installDir/usr/include/c++
log_exec mv $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION}/include $installDir/usr/include/gcc
log_exec mv $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION}/include-fixed $installDir/usr/include/gcc/fixed

# Consolidate includes in /usr/include for kernel compilation
for include in $(cd $installDir/usr/include/gcc; find . -type f); do
	include="${include#./}"
	subdir=$(dirname $include)
	if [[ " $subdir" =~ " fixed" ]]; then continue; fi
	if [ ! -r "$installDir/usr/include/$include" ]; then
		log_exec mkdir -p "$installDir/usr/include/$subdir"
		log_exec mv "$installDir/usr/include/gcc/$include" "$installDir/usr/include/$subdir"
	elif cmp -s "$installDir/usr/include/gcc/$include" "$installDir/usr/include/$include"; then
		log_exec rm -f "$installDir/usr/include/gcc/$include"
	fi
done
for subdir in $(cd $installDir/usr/include/gcc; find . -type d); do
	if [ "$subdir" = "." ]; then continue; fi
	if is_tree_empty "$installDir/usr/include/gcc/$subdir"; then
		log_exec rm -rf "$installDir/usr/include/gcc/$subdir"
	fi
done
if is_tree_empty "$installDir/usr/include/gcc"; then
	log_exec rm -rf "$installDir/usr/include/gcc"
fi

# Remove native GCC components (already copied to $nativeInstallDir)
log_exec rm -rf $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION}/finclude
log_exec rm -rf $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION}/install-tools
log_exec rm -rf $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION}/plugin

# Copy GCC-provided library files (already copied to $nativeInstallDir)
log_exec mv $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION}/*.o $installDir/usr/lib
log_exec mv $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION} $installDir/usr/lib/gcc

# Even empty, the lib/gcc/$compiler_prefix/${GCC_VERSION} directory is needed,
# as gcc will go up&down through this directory tree to find its sub-tools...
# To be sure we create a "marker" file there
log_exec mkdir -p $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION}
log_exec echo "This directory is needed, even if empty" >  $SRC_NAME/lib/gcc/$compiler_prefix/${GCC_VERSION}/needed-empty-directory

# Install the cross compiler itself

log "Install the bare-metal compiler itself"
copy_tree_excluding -d $SRC_NAME $nativePrefix/usr

log "And clean up the work environment"
log_exec rm -rf $SRC_NAME
